/*-----------------------------------------------------------------------------

  Node.h
  SpinParserLib
  By Dave Fletcher
  (c) 2008 All Rights Reserved

  This code is released under the terms of the GNU Lesser General Public
  License (LGPL). See COPYING.

  INDUSTRIAL APPLICATION WARNING:
  Please note, this software is EXPERIMENTAL and is likely to remain in this
  state for a good while. Please do not use this software in mission critical
  applications and in situations where the safety of human-beings is at stake.

  This document is non-normative. It is not compared with the original Parallax
  documents or any Parallax source codes. It is derived mainly from the user
  manual and tests run inside the Propeller Tool.

  If you find bugs, the current place to report them is in the Parallax user
  forums. This may change in future:

    http://forums.parallax.com/forums/?f=25&m=253050&g=253340

-----------------------------------------------------------------------------*/

#ifndef SP_NODE
#define SP_NODE

#include <Pointer.h>
#include <StringPointer.h>

namespace SpinParser {

  class Node {

    public:

      /**
       * Pointer to a node.
       */
      typedef Pointer<Node> NodePointer;

      /**
       * List of Node pointers.
       */
      typedef std::list<NodePointer> NodeList;

     virtual ~Node(void);

	 /**
       * Get the type name of this node.
       * @return Node type name.
       */
      virtual StringPointer getType(void);

      /**
       * Get the unique identifer for this node.
       * @return Unique identifer for this node.
       */
      virtual StringPointer getId(void);

      /**
       * Create a list of nodes that match the specified query.
       * ( documentation coming later )
       * @return List of nodes.
       */
      virtual NodeList query(StringPointer query);

      /**
       * Get the number of child nodes if this node is nonterminal, or zero
       * otherwise.
       * @return The number of children.
       */
      virtual size_t getChildCount(void);

      /**
       * Get the number of child nodes if this node is nonterminal, or zero
       * otherwise.
       * @return The child at the specified index, or a INVALID type node if
       *         index is out of range.
       */
      virtual NodePointer getChild(size_t index);

      /**
       * Convienience function to detect if this node has child nodes.
       * @return True if the Node has children, false otherwise.
       */
      bool hasChildren(void);

  };

}

#endif /* [SP_NODE] */
