/*-----------------------------------------------------------------------------

  Tree.h
  SpinParserLib
  By Dave Fletcher
  (c) 2008 All Rights Reserved

  This code is released under the terms of the GNU Lesser General Public
  License (LGPL). See COPYING.

  INDUSTRIAL APPLICATION WARNING:
  Please note, this software is EXPERIMENTAL and is likely to remain in this
  state for a good while. Please do not use this software in mission critical
  applications and in situations where the safety of human-beings is at stake.

  This document is non-normative. It is not compared with the original Parallax
  documents or any Parallax source codes. It is derived mainly from the user
  manual and tests run inside the Propeller Tool.

  If you find bugs, the current place to report them is in the Parallax user
  forums. This may change in future:

    http://forums.parallax.com/forums/?f=25&m=253050&g=253340

-----------------------------------------------------------------------------*/

#ifndef SP_TREE
#define SP_TREE

#include <map>
#include <list>
#include <Node.h>
#include <Pointer.h>
#include <StringPointer.h>

namespace SpinParser {

  class Tree {

    public:

      typedef std::list<Node::NodePointer> ObjectList;
      typedef ObjectList::iterator ObjectListIterator;

      /**
       * A SpinParser Tree is a collection of objects. The top level object is
       * named <<top>> and is always available after successful parsing.
       * @param name Name of the desired object.
       * @return A list of Pointers. Each named object is potentially an array
       * of objects. The <<top>> object list will only contain one object.
       */
      ObjectList getObjects(StringPointer name);

      /**
       * Convienience function. Synonym for `getObject("<<top>>")->front()`.
       * @return A pointer to the root node of the top object.
       */
      Node::NodePointer getTopObject(void);

      /**
       * Gets the names of all objects in this Tree. Each can be passed to
       * getObject() and will receive a Nonterminal Node.
       * @return An iterator over the list of available object names.
       */
      StringList &getObjectNames(void);

      /**
       * Parser calls this to add objects.
       * @param name Name of the object as given.
       * @param root The tree node root of the object.
       */
      void addObject(StringPointer name, Node::NodePointer root);
      
    private:
      typedef std::map<StringPointer, ObjectList, ltstr> ObjectMap;
      typedef ObjectMap::iterator ObjectMapIterator;
      StringList objectnames;
      ObjectMap children;
  };

}

#endif /* [SP_TREE] */
